/*
 * This file is a part of TTCalc - a mathematical calculator
 * and is distributed under the (new) BSD licence.
 * Author: Tomasz Sowa <t.sowa@slimaczek.pl>
 */

/* 
 * Copyright (c) 2006-2009, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *    
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *    
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "compileconfig.h"
#include "languages.h"




const char * Languages::ErrorMessage(Country country, ttmath::ErrorCode code)
{
const char * unk_err = "unknown error";

	if( error_messages_tab.empty() )
		InitErrorMessagesTab();

	unsigned int cid = static_cast<unsigned int>( country );
	if( cid >= error_messages_tab.size() )
		return unk_err;

	std::map<ttmath::ErrorCode, std::string>::const_iterator i;

	i = error_messages_tab[cid].find( code );
	if( i == error_messages_tab[cid].end() )
		return unk_err;

return i->second.c_str();
}


const char * Languages::ErrorMessage(ttmath::ErrorCode code)
{
	return ErrorMessage(current_country, code);
}



void Languages::InsertErrorPair(ttmath::ErrorCode code, const char * message)
{
	if( error_messages_tab.empty() )
		return;

	error_messages_tab.back().insert( std::make_pair(code, message) );
}


void Languages::InitErrorMessagesTab()
{
	error_messages_tab.push_back( std::map<ttmath::ErrorCode, std::string>() );

	InsertErrorPair(ttmath::err_ok,"ok");
	InsertErrorPair(ttmath::err_nothing_has_read,"");
	InsertErrorPair(ttmath::err_unknown_character,"An unknown character");
	InsertErrorPair(ttmath::err_unexpected_final_bracket,"An unexpected final bracket");
	InsertErrorPair(ttmath::err_stack_not_clear,"An unknown character has left");
	InsertErrorPair(ttmath::err_unknown_variable,"An unknown variable");
	InsertErrorPair(ttmath::err_division_by_zero,"Division by zero");
	InsertErrorPair(ttmath::err_interrupt,"The calculating has been broken");
	InsertErrorPair(ttmath::err_overflow,"Overflow");
	InsertErrorPair(ttmath::err_unknown_function,"An unknown function");
	InsertErrorPair(ttmath::err_unknown_operator,"An unknown operator");
	InsertErrorPair(ttmath::err_unexpected_semicolon_operator,"An unexpected semicolon operator");
	InsertErrorPair(ttmath::err_improper_amount_of_arguments,"Improper amount of arguments");
	InsertErrorPair(ttmath::err_improper_argument,"Improper argument");
	InsertErrorPair(ttmath::err_unexpected_end,"Unexpected end");
	InsertErrorPair(ttmath::err_internal_error,"An internal error");

	InsertErrorPair(ttmath::err_incorrect_name,"Incorrect name of a variable or function");
	InsertErrorPair(ttmath::err_incorrect_value,"Incorrect value of a variable or function");
	InsertErrorPair(ttmath::err_variable_exists,"This variable already exists");
	InsertErrorPair(ttmath::err_variable_loop,"There's a recurrence between variables");
	InsertErrorPair(ttmath::err_functions_loop,"There's a recurrence between functions");
	InsertErrorPair(ttmath::err_must_be_only_one_value,"Variables or functions must return only one value");
	InsertErrorPair(ttmath::err_still_calculating,"Calculating...");
	InsertErrorPair(ttmath::err_too_big_factorial,"A too big argument for the factorial() function");

// pl

	error_messages_tab.push_back( std::map<ttmath::ErrorCode, std::string>() );

	InsertErrorPair(ttmath::err_ok,"ok");
	InsertErrorPair(ttmath::err_nothing_has_read,"");
	InsertErrorPair(ttmath::err_unknown_character,"Nieznany znak");
	InsertErrorPair(ttmath::err_unexpected_final_bracket,"Nieoczekiwany nawias zamykajcy");
	InsertErrorPair(ttmath::err_stack_not_clear,"Pozosta nieznany znak");
	InsertErrorPair(ttmath::err_unknown_variable,"Nieznana zmienna");
	InsertErrorPair(ttmath::err_division_by_zero,"Dzielenie przez zero");
	InsertErrorPair(ttmath::err_interrupt,"Obliczenia zostay przerwane");
	InsertErrorPair(ttmath::err_overflow,"Przekroczony zakres");
	InsertErrorPair(ttmath::err_unknown_function,"Nieznana funkcja");
	InsertErrorPair(ttmath::err_unknown_operator,"Nieznany operator");
	InsertErrorPair(ttmath::err_unexpected_semicolon_operator,"Nieoczekiwany operator 'rednik'");
	InsertErrorPair(ttmath::err_improper_amount_of_arguments,"Niewaciwa liczba argumentw");
	InsertErrorPair(ttmath::err_improper_argument,"Niewaciwy argument");
	InsertErrorPair(ttmath::err_unexpected_end,"Nieoczekiwany koniec");
	InsertErrorPair(ttmath::err_internal_error,"Bd wewntrzny programu!");

	InsertErrorPair(ttmath::err_incorrect_name,"Nieprawidowa nazwa zmiennej lub funkcji");
	InsertErrorPair(ttmath::err_incorrect_value,"Nieprawidowa warto zmiennej lub funkcji");
	InsertErrorPair(ttmath::err_variable_exists,"Ta zmienna juz istnieje");
	InsertErrorPair(ttmath::err_variable_loop,"Pomidzy zmiennymi zachodzi wywoanie rekurencyjne");
	InsertErrorPair(ttmath::err_functions_loop,"Pomidzy funkcjami zachodzi wywoanie rekurencyjne");
	InsertErrorPair(ttmath::err_must_be_only_one_value,"Zmienne albo funkcje mog posiada (zwraca) tylko jedn warto");
	InsertErrorPair(ttmath::err_still_calculating,"Obliczanie...");
	InsertErrorPair(ttmath::err_too_big_factorial,"Zbyt duy argument dla funkcji factorial()");


// sp

	error_messages_tab.push_back( std::map<ttmath::ErrorCode, std::string>() );
	
	InsertErrorPair(ttmath::err_ok,"ok");
	InsertErrorPair(ttmath::err_nothing_has_read,"");
	InsertErrorPair(ttmath::err_unknown_character,"Un caracter desconocido");
	InsertErrorPair(ttmath::err_unexpected_final_bracket,"corchete final inesperado");
	InsertErrorPair(ttmath::err_stack_not_clear,"caracter desconocido a la izquierda");
	InsertErrorPair(ttmath::err_unknown_variable,"Variable desconocida");
	InsertErrorPair(ttmath::err_division_by_zero,"Division por cero");
	InsertErrorPair(ttmath::err_interrupt,"El calculo se ha colapsado");
	InsertErrorPair(ttmath::err_overflow,"Overflow");
	InsertErrorPair(ttmath::err_unknown_function,"Funcion desconocida");
	InsertErrorPair(ttmath::err_unknown_operator,"Operador desconocida");
	InsertErrorPair(ttmath::err_unexpected_semicolon_operator,"punto y coma inesperado");
	InsertErrorPair(ttmath::err_improper_amount_of_arguments,"numero de argumentos incorrecto");
	InsertErrorPair(ttmath::err_improper_argument,"argumento incorrecto");
	InsertErrorPair(ttmath::err_unexpected_end,"Fin inexperado");
	InsertErrorPair(ttmath::err_internal_error,"Error interno");

	InsertErrorPair(ttmath::err_incorrect_name,"nombre incorrecto de variable o funcion");
	InsertErrorPair(ttmath::err_incorrect_value,"valor incorrecto de variable o funcion");
	InsertErrorPair(ttmath::err_variable_exists,"Variable ya existe");
	InsertErrorPair(ttmath::err_variable_loop,"Recurrencia entre variables");
	InsertErrorPair(ttmath::err_functions_loop,"Recurrencia entre funciones");
	InsertErrorPair(ttmath::err_must_be_only_one_value,"Variables o funciones solo pueden retornar un valor");
	InsertErrorPair(ttmath::err_still_calculating,"Calculando ...");
	InsertErrorPair(ttmath::err_too_big_factorial,"Argumento muy grande para la funcion factorial()");

}



// for gui messages




const char * Languages::GuiMessage(Country country, GuiMsg code)
{
const char * unk_msg = "unknown";

	if( gui_messages_tab.empty() )
		InitGuiMessagesTab();

	unsigned int cid = static_cast<unsigned int>( country );
	if( cid >= gui_messages_tab.size() )
		return unk_msg;

	std::map<GuiMsg, std::string>::const_iterator i;

	i = gui_messages_tab[cid].find( code );
	if( i == gui_messages_tab[cid].end() )
		return unk_msg;

return i->second.c_str();
}



const char * Languages::GuiMessage(GuiMsg code)
{
	return GuiMessage(current_country, code);
}



void Languages::InsertGuiPair(GuiMsg code, const char * message)
{
	if( gui_messages_tab.empty() )
		return;

	gui_messages_tab.back().insert( std::make_pair(code, message) );
}



void Languages::InitGuiMessagesTab()
{
	// en
	gui_messages_tab.push_back( std::map<GuiMsg, std::string>() );

	InsertGuiPair(button_ok,"Ok");
	InsertGuiPair(button_cancel,"Cancel");
	InsertGuiPair(message_box_caption,"TTCalc");

	InsertGuiPair(dialog_box_add_variable_caption,"Add a new variable");
	InsertGuiPair(dialog_box_edit_variable_caption,"Edit a variable");
	InsertGuiPair(dialog_box_add_variable_incorrect_name,"An incorrect name of the variable");
	InsertGuiPair(dialog_box_add_variable_incorrect_value,"An incorrect value of the variable");
	InsertGuiPair(dialog_box_add_variable_variable_exists,"This variable already exists");
	InsertGuiPair(dialog_box_edit_variable_unknown_variable,"There isn't this variable in my table. There's probably an internal error!");
	InsertGuiPair(dialog_box_delete_variable_confirm,"Do you want to delete these variables?");
	InsertGuiPair(dialog_box_variable_not_all_deleted,"There are some variables which I was not able to delete. Probably an internal error!");
	InsertGuiPair(dialog_box_add_function_caption,"Add a new function");
	InsertGuiPair(dialog_box_edit_function_caption,"Edit a function");
	InsertGuiPair(dialog_box_add_function_function_exists,"This function already exists");
	InsertGuiPair(dialog_box_edit_function_unknown_function,"There isn't this function in my table. There's probably an internal error!");
	InsertGuiPair(dialog_box_delete_function_confirm,"Do you want to delete these functions?");
	InsertGuiPair(dialog_box_function_not_all_deleted,"There are some functions which I was not able to delete. Probably an internal error!");
	InsertGuiPair(dialog_box_add_function_incorrect_name,"An incorrect name of the function");

	InsertGuiPair(list_variables_header_1,"Name");
	InsertGuiPair(list_variables_header_2,"Value");
	InsertGuiPair(list_functions_header_1,"Name");
	InsertGuiPair(list_functions_header_2,"Param.");
	InsertGuiPair(list_functions_header_3,"Value");

	InsertGuiPair(button_add,"Add");
	InsertGuiPair(button_edit,"Edit");
	InsertGuiPair(button_delete,"Delete");
	InsertGuiPair(button_clear,"C");
	InsertGuiPair(tab_standard,"Standard");
	InsertGuiPair(tab_variables,"Variables");
	InsertGuiPair(tab_functions,"Functions");
	InsertGuiPair(tab_precision,"Precision");
	InsertGuiPair(tab_display,"Display");
	InsertGuiPair(tab_convert,"Convert");

	InsertGuiPair(radio_precision_1,"Small - 96 bits for the mantissa, 32 bits for the exponent");
	InsertGuiPair(radio_precision_2,"Medium - 288 bits for the mantissa, 64 bits for the exponent");
	InsertGuiPair(radio_precision_3,"Big - 864 bits for the mantissa, 128 bits for the exponent");


	InsertGuiPair(precision_1_info,"(+/-)6.9 e+646457021, 26 valid digits (decimal)");
	InsertGuiPair(precision_2_info,"(+/-)3.4e+2776511644261678652, 85 valid digits");
	InsertGuiPair(precision_3_info,"(+/-)6.7e+51217599719369681875006054625051616609, 258 digits");

	InsertGuiPair(overflow_during_printing,"Overflow during printing");

	InsertGuiPair(combo_rounding_none,"None");
	InsertGuiPair(combo_rounding_integer,"to integer");
	InsertGuiPair(combo_rounding_to_number,"to");
	InsertGuiPair(combo_rounding_after_comma,"digit(s)");
	InsertGuiPair(check_remove_zeroes,"Remove last trailing zeroes");

	InsertGuiPair(display_input,			"Input");
	InsertGuiPair(display_output,			"Output");
	InsertGuiPair(display_rounding,			"Rounding");
	InsertGuiPair(display_always_scientific,"Always");
	InsertGuiPair(display_not_always_scientific,"When the exponent is greater than:");
	InsertGuiPair(display_group_scientific,	"Print the result as the scientific value");
	InsertGuiPair(display_decimal_point,	"Decimal point");
	InsertGuiPair(display_deg_red_grad,		"sin/cos asin/acos");
	InsertGuiPair(convert_type,				"Type");
	InsertGuiPair(convert_input,			"Input");
	InsertGuiPair(convert_output,			"Output");
	InsertGuiPair(convert_dynamic_output,	"Auto prefix");

	InsertGuiPair(menu_view,				"&View");
	InsertGuiPair(menu_edit,				"&Edit");
	InsertGuiPair(menu_help,				"&Help");
	InsertGuiPair(menu_language,			"&Language");
	InsertGuiPair(menu_view_new_window,		"&New window");
	InsertGuiPair(menu_view_normal_view,	"No&rmal view");
	InsertGuiPair(menu_view_compact_view,	"C&ompact view");
	InsertGuiPair(menu_view_always_on_top,	"&Always on top");
	InsertGuiPair(menu_view_lang_english,	"&English");
	InsertGuiPair(menu_view_lang_polish,	"&Polish");
	InsertGuiPair(menu_view_lang_spanish,	"&Spanish");
	InsertGuiPair(menu_view_close_program,	"&Close");
	InsertGuiPair(menu_edit_undo,			"&Undo");
	InsertGuiPair(menu_edit_paste,			"&Paste");
	InsertGuiPair(menu_edit_copy_result,	"&Copy the result");
	InsertGuiPair(menu_help_help,			"&Help");
	InsertGuiPair(menu_help_project_page,	"&Project page");
	InsertGuiPair(menu_help_about,			"&About");
	InsertGuiPair(cant_init_calculations,	"I could not initialize the module for calculations");
	InsertGuiPair(message_box_error_caption,"TTCalc");
	InsertGuiPair(cant_create_thread,		"I could not create the second thread for calculating");
	InsertGuiPair(cant_create_main_window,	"I could not create the main window of the application");
	InsertGuiPair(cant_init_common_controls,"I could not initialize the common controls (InitCommonControlsEx)");
	InsertGuiPair(about_text,
		"Mathematical calculator TTCalc %d.%d.%d%s%s\r\n"
		"Author: Tomasz Sowa\r\n"
		"Contact: t.sowa@slimaczek.pl\r\n"
		"Licence: BSD (open source)\r\n"
		"Project page: http://ttcalc.sourceforge.net\r\n"
		"Bignum library: TTMath %d.%d.%d%s\r\n"
		"Programming language: C++\r\n"
		"Compiler: %s\r\n"
		"%s"	// for upx
		"\r\n"
		"TTCalc uses the TTMath bignum library"
		" which can be found at http://sourceforge.net/projects/ttmath\r\n"
		"\r\n"
#ifdef TTCALC_PORTABLE
		"This is the portable version of the program TTCalc. In this version "
		"you can calculate only with one kind of precision (96 bits for the "
		"mantissa and 32 bits for the exponent) it's about +/-6.9e+646457021.\r\n"
		"\r\n"
#endif
		"If you have any questions, advices or interesting ideas about"
		" this program or if you want to join to this project as"
		" a developer or a programmer feel free to contant with the author."
		);
	InsertGuiPair(about_text_portable_version, " portable version");
	InsertGuiPair(about_text_exe_packer,	"EXE Packer: UPX 3.03\r\n");
	InsertGuiPair(about_box_title,			"About");
	InsertGuiPair(about_box_button_close,	"Close");
	InsertGuiPair(unknown_error,			"An unknown error has occurred");
	InsertGuiPair(cant_find_help,			"I can't find any help files");
	InsertGuiPair(cant_open_project_page,   "I can't open the project webpage");

	// pl
	gui_messages_tab.push_back( std::map<GuiMsg, std::string>() );

	InsertGuiPair(button_ok,"Ok");
	InsertGuiPair(button_cancel,"Anuluj");
	InsertGuiPair(message_box_caption,"TTCalc");

	InsertGuiPair(dialog_box_add_variable_caption,"Dodaj now zmienn");
	InsertGuiPair(dialog_box_edit_variable_caption,"Zmie warto zmiennej");
	InsertGuiPair(dialog_box_add_variable_incorrect_name,"Nie prawidowa nazwa zmiennej");
	InsertGuiPair(dialog_box_add_variable_incorrect_value,"Nie prawidowa warto zmiennej");
	InsertGuiPair(dialog_box_add_variable_variable_exists,"Podana zmienna ju istnieje");
	InsertGuiPair(dialog_box_edit_variable_unknown_variable,"Podanej zmiennej nie ma w tablicy. Prawdopodobnie bd wewntrzny programu.");
	InsertGuiPair(dialog_box_delete_variable_confirm,"Czy napewno usun zaznaczone zmienne?");
	InsertGuiPair(dialog_box_variable_not_all_deleted,"Zostao kilka zmiennych ktrych nie mona byo skasowa. Prawdopodobnie bd wewntrzny programu.");
	InsertGuiPair(dialog_box_add_function_caption,"Dodaj now funkcj");
	InsertGuiPair(dialog_box_edit_function_caption,"Zmie warto funkcji");
	InsertGuiPair(dialog_box_add_function_function_exists,"Podana funkcja ju istnieje");
	InsertGuiPair(dialog_box_edit_function_unknown_function,"Podanej funkcji nie ma w tablicy. Prawdopodobnie bd wewntrzny programu.");
	InsertGuiPair(dialog_box_delete_function_confirm,"Czy napewno usun zaznaczone funkcje?");
	InsertGuiPair(dialog_box_function_not_all_deleted,"Zostao kilka funkcji ktrych nie mona byo skasowa. Prawdopodobnie bd wewntrzny programu.");
	InsertGuiPair(dialog_box_add_function_incorrect_name,"Nieprawidowa nazwa funkcji");


	InsertGuiPair(list_variables_header_1,"Nazwa");
	InsertGuiPair(list_variables_header_2,"Warto");
	InsertGuiPair(list_functions_header_1,"Nazwa");
	InsertGuiPair(list_functions_header_2,"Parametry");
	InsertGuiPair(list_functions_header_3,"Warto");

	InsertGuiPair(button_add,"Dodaj");
	InsertGuiPair(button_edit,"Edytuj");
	InsertGuiPair(button_delete,"Usu");
	InsertGuiPair(button_clear,"C");
	InsertGuiPair(tab_standard,"Standard");
	InsertGuiPair(tab_variables,"Zmienne");
	InsertGuiPair(tab_functions,"Funkcje");
	InsertGuiPair(tab_precision,"Precyzja");
	InsertGuiPair(tab_display,"Wywietlanie");
	InsertGuiPair(tab_convert,"Konwersja");

	InsertGuiPair(radio_precision_1,"Maa - 96 bitowa mantysa, 32 bitowy wykadnik");
	InsertGuiPair(radio_precision_2,"rednia - 288 bitowa mantysa, 64 bitowy wykadnik");
	InsertGuiPair(radio_precision_3,"Dua - 864 bitowa mantysa, 128 bitowy wykadnik");

	InsertGuiPair(precision_1_info,"(+/-)6.9e+646457021, 26 cyfr znaczcych (w rozwiniciu dziesitnym)");
	InsertGuiPair(precision_2_info,"(+/-)3.4e+2776511644261678652, 85 cyfr znaczcych");
	InsertGuiPair(precision_3_info,"(+/-)6.7e+51217599719369681875006054625051616609, 258 cyfr");

	InsertGuiPair(overflow_during_printing,"Przepenienie podczas wypisywania");

	InsertGuiPair(combo_rounding_none,"bez zmian");
	InsertGuiPair(combo_rounding_integer,"do cakowitej");
	InsertGuiPair(combo_rounding_to_number,"do");
	InsertGuiPair(combo_rounding_after_comma,"cyfr(y)");
	InsertGuiPair(check_remove_zeroes,"Skasuj nieznaczce zera");

	InsertGuiPair(display_input,"Wejcie");
	InsertGuiPair(display_output,"Wyjcie");
	InsertGuiPair(display_rounding,"Zaokrglenie");
	InsertGuiPair(display_always_scientific,"Zawsze");
	InsertGuiPair(display_not_always_scientific,"Jeli eksponent jest wikszy ni:");
	InsertGuiPair(display_group_scientific,	"Wywietl wynik w postaci naukowej");
	InsertGuiPair(display_decimal_point,	"Przecinek dziesitny");
	InsertGuiPair(display_deg_red_grad,		"sin/cos asin/acos");
	InsertGuiPair(convert_type,				"Typ");
	InsertGuiPair(convert_input,			"Wejcie");
	InsertGuiPair(convert_output,			"Wyjcie");
	InsertGuiPair(convert_dynamic_output,	"Automatyczny prefiks");

	InsertGuiPair(menu_view,				"&Widok");
	InsertGuiPair(menu_edit,				"&Edycja");
	InsertGuiPair(menu_help,				"&Pomoc");
	InsertGuiPair(menu_language,			"&Jzyk");
	InsertGuiPair(menu_view_new_window,		"&Nowe okno");
	InsertGuiPair(menu_view_normal_view,	"Widok no&rmalny");
	InsertGuiPair(menu_view_compact_view,	"Widok &kompaktowy");
	InsertGuiPair(menu_view_always_on_top,	"Zawsze na &wierzchu");
	InsertGuiPair(menu_view_lang_english,	"&Angielski");
	InsertGuiPair(menu_view_lang_polish,	"&Polski");
	InsertGuiPair(menu_view_lang_spanish,	"&Hiszpaski");
	InsertGuiPair(menu_view_close_program,	"&Zamknij");
	InsertGuiPair(menu_edit_undo,			"&Cofnij");
	InsertGuiPair(menu_edit_paste,			"&Wklej");
	InsertGuiPair(menu_edit_copy_result,	"&Kopiuj wynik");
	InsertGuiPair(menu_help_help,			"&Pomoc");
	InsertGuiPair(menu_help_project_page,	"&Strona projektu");
	InsertGuiPair(menu_help_about,			"&O programie");
	InsertGuiPair(cant_init_calculations,	"Nie udao si zainicjalizowa moduu obsugi oblicze");
	InsertGuiPair(message_box_error_caption,"TTCalc");
	InsertGuiPair(cant_create_thread,		"Nie udao si utworzy drugiego wtku do oblicze");
	InsertGuiPair(cant_create_main_window,	"Nie udao si utworzy gwnego okna aplikacji");
	InsertGuiPair(cant_init_common_controls,"Nie udao si zainicjalizowa obsugi Common Controls (InitCommonControlsEx)");
	InsertGuiPair(about_text,
		"Kalkulator matematyczny TTCalc %d.%d.%d%s%s\r\n"
		"Autor: Tomasz Sowa\r\n"
		"Kontakt: t.sowa@slimaczek.pl\r\n"
		"Licencja: BSD (open source)\r\n"
		"Strona projektu: http://ttcalc.sourceforge.net\r\n"
		"Biblioteka duych liczb: TTMath %d.%d.%d%s\r\n"
		"Jzyk programowania: C++\r\n"
		"Kompilator: %s\r\n"
		"%s"	// for upx
		"\r\n"
		"Ten program uywa biblioteki duych liczb TTMath"
		" ktra jest dostpna na http://sourceforge.net/projects/ttmath\r\n"
		"\r\n"
#ifdef TTCALC_PORTABLE
		"To jest wersja portable programu TTCalc. W tej wersji moesz dokonywa "
		"oblicze jedynie z jednym rodzajem precyzji (96 bitowa mantysa "
		"oraz 32 bitowy wykadnik) to jest okoo +/-6.9e+646457021.\r\n"
		"\r\n"
#endif
		"Jeeli masz jakie pytania, rady, ciekawe pomysy dotyczce"
		" tego programu lub chciaby doczy jako projektant/programista"
		" poprostu skontaktuj si z autorem."
		);
	InsertGuiPair(about_text_portable_version, " wersja portable");
	InsertGuiPair(about_text_exe_packer,	"Paker exe: UPX 3.03\r\n");
	InsertGuiPair(about_box_title,			"O programie");
	InsertGuiPair(about_box_button_close,	"Zamknij");
	InsertGuiPair(unknown_error,			"Nieznany kod bdu");
	InsertGuiPair(cant_find_help,			"Nie mog znale adnych plikw pomocy");
	InsertGuiPair(cant_open_project_page,   "Nie mog otworzy strony projektu");


// sp

	gui_messages_tab.push_back( std::map<GuiMsg, std::string>() );

	InsertGuiPair(button_ok,"Ok");
	InsertGuiPair(button_cancel,"Cancelar");
	InsertGuiPair(message_box_caption,"TTCalc");

	InsertGuiPair(dialog_box_add_variable_caption,"Aadir nueva variable");
	InsertGuiPair(dialog_box_edit_variable_caption,"Editar variable");
	InsertGuiPair(dialog_box_add_variable_incorrect_name,"Nombre incorrecto de variable");
	InsertGuiPair(dialog_box_add_variable_incorrect_value,"Valor incorrecto de variable");
	InsertGuiPair(dialog_box_add_variable_variable_exists,"Esta variable ya existe");
	InsertGuiPair(dialog_box_edit_variable_unknown_variable,"No hay variable en mi tabla!. Seguramente haya un error interno!");
	InsertGuiPair(dialog_box_delete_variable_confirm,"Quiere borrar las variables internas?");
	InsertGuiPair(dialog_box_variable_not_all_deleted,"Hay algunas variables que no se pueden borrar. Probablemente hay un error interno!");
	InsertGuiPair(dialog_box_add_function_caption,"Aadir funcion");
	InsertGuiPair(dialog_box_edit_function_caption,"Editar funcion");
	InsertGuiPair(dialog_box_add_function_function_exists,"Esta funcion ya existe");
	InsertGuiPair(dialog_box_edit_function_unknown_function,"No existe esta funcion en mi tabla. Problablemente error interno!");
	InsertGuiPair(dialog_box_delete_function_confirm,"Quiere borrar estas funciones?");
	InsertGuiPair(dialog_box_function_not_all_deleted,"Hay algunas funciones que no se pueden borrar. Probablmenete error interno!");
	InsertGuiPair(dialog_box_add_function_incorrect_name,"Nombre incorrecto de la funcion");

	InsertGuiPair(list_variables_header_1,"Nombre");
	InsertGuiPair(list_variables_header_2,"Valor");
	InsertGuiPair(list_functions_header_1,"Nombre");
	InsertGuiPair(list_functions_header_2,"Param.");
	InsertGuiPair(list_functions_header_3,"Valor");

	InsertGuiPair(button_add,"Aadir");
	InsertGuiPair(button_edit,"Editar");
	InsertGuiPair(button_delete,"Borrar");
	InsertGuiPair(button_clear,"C");
	InsertGuiPair(tab_standard,"Standard");
	InsertGuiPair(tab_variables,"Variables");
	InsertGuiPair(tab_functions,"Funciones");
	InsertGuiPair(tab_precision,"Precision");
	InsertGuiPair(tab_display,"Pantalla");
	InsertGuiPair(tab_convert,"Convertir");

	InsertGuiPair(radio_precision_1,"Pequeo - 96 bits para la mantissa, 32 bits para el exponente");
	InsertGuiPair(radio_precision_2,"Mediano - 288 bits para la mantissa, 64 bits para el exponente");
	InsertGuiPair(radio_precision_3,"Grande - 864 bits para la mantissa, 128 bits para el exponente");


	InsertGuiPair(precision_1_info,"(+/-)6.9 e+646457021, 26 digitos validos (decimal)");
	InsertGuiPair(precision_2_info,"(+/-)3.4e+2776511644261678652, 85 digitos validos");
	InsertGuiPair(precision_3_info,"(+/-)6.7e+51217599719369681875006054625051616609, 258 digitos");

	InsertGuiPair(overflow_during_printing,"Sobredimensin durante impresin");

	InsertGuiPair(combo_rounding_none,"Ninguno");
	InsertGuiPair(combo_rounding_integer,"a entero");
	//InsertGuiPair(combo_rounding_to_number,"a numero");
	// this text was too long (the combobox is shorter now) -- I'm not sure whether this is correct now... (I don't know spanish at all)
	InsertGuiPair(combo_rounding_to_number,"a");
	InsertGuiPair(combo_rounding_after_comma,"digito(s)");
	InsertGuiPair(check_remove_zeroes,"Eliminar ceros no-significativos");

	InsertGuiPair(display_input,			"Entrada");
	InsertGuiPair(display_output,			"Salida");
	InsertGuiPair(display_rounding,			"Redondeo");
	InsertGuiPair(display_always_scientific,"Siempre");
	InsertGuiPair(display_not_always_scientific,"Cuando el exponente es mas grande que:");
	InsertGuiPair(display_group_scientific,	"Imprimir el resultado como valor cientifico");
	InsertGuiPair(display_decimal_point,	"Decimal point");
	InsertGuiPair(display_deg_red_grad,		"sin/cos asin/acos");
	InsertGuiPair(convert_type,				"Tipo");
	InsertGuiPair(convert_input,			"Entrada");
	InsertGuiPair(convert_output,			"Salida");
	InsertGuiPair(convert_dynamic_output,	"Auto prefijo");

	InsertGuiPair(menu_view,				"&Ver");
	InsertGuiPair(menu_edit,				"&Editar");
	InsertGuiPair(menu_help,				"&Ayuda");
	InsertGuiPair(menu_language,			"&Idioma");
	InsertGuiPair(menu_view_new_window,		"&Nueva ventana");
	InsertGuiPair(menu_view_normal_view,	"Vista No&rmal");
	InsertGuiPair(menu_view_compact_view,	"Vista C&ompacta");
	InsertGuiPair(menu_view_always_on_top,	"&Siempre arriba");
	InsertGuiPair(menu_view_lang_english,	"&Ingles");
	InsertGuiPair(menu_view_lang_polish,	"&Polaco");
	InsertGuiPair(menu_view_lang_spanish,	"&Espaol");
	InsertGuiPair(menu_view_close_program,	"&Close");
	InsertGuiPair(menu_edit_undo,			"&Deshacer");
	InsertGuiPair(menu_edit_paste,			"&Pegar");
	InsertGuiPair(menu_edit_copy_result,	"&Copiar resultado");
	InsertGuiPair(menu_help_help,			"&Ayuda");
	InsertGuiPair(menu_help_project_page,	"&Pagina del proyecto");
	InsertGuiPair(menu_help_about,			"&Acerca de");
	InsertGuiPair(cant_init_calculations,	"No se puede inicializar el modulo de calculo");
	InsertGuiPair(message_box_error_caption,"TTCalc");
	InsertGuiPair(cant_create_thread,		"No se puede crear la segunda hebra para calcular");
	InsertGuiPair(cant_create_main_window,	"No se puede crear la ventana principal par ala aplicacion");
	InsertGuiPair(cant_init_common_controls,"No se pueden inicializar los controles comunes (InitCommonControlsEx)");
	InsertGuiPair(about_text,
		"Calculadora matematica TTCalc %d.%d.%d%s%s\r\n"
		"Autor: Tomasz Sowa\r\n"
		"Contacto: t.sowa@slimaczek.pl\r\n"
		"Licencia: BSD (open source)\r\n"
		"Pagina del proyecto: http://ttcalc.sourceforge.net\r\n"
		"Biblioteca Bignum: TTMath %d.%d.%d%s\r\n"
		"Lenguaje de programacion: C++\r\n"
		"Compilador: %s\r\n"
		"%s"	// for upx
		"\r\n"
		"TTCalc usa la biblioteca TTMath bignum"
		" la cual puede ser encontrada en http://sourceforge.net/projects/ttmath\r\n"
		"\r\n"
#ifdef TTCALC_PORTABLE
		"Esta es la version portatil del programa TTCalc. En esta version "
		"se puede calcular con una precision tal que (96 bits para la "
		"mantisa y 32 bits para el exponente)  que es aproximadamente +/-6.9e+646457021.\r\n"
		"\r\n"
#endif
		"Si tienes preguntas, consejos o ideas interesantes sobre"
		" este programa o si quieres unirte a este proyecto como"
		" desarrollador o programador, no dudes en contactar con el autor."
		);
	InsertGuiPair(about_text_portable_version, " version portatil");
	InsertGuiPair(about_text_exe_packer,	"EXE Packer: UPX 3.03\r\n");
	InsertGuiPair(about_box_title,			"Acerca de");
	InsertGuiPair(about_box_button_close,	"Cerrar");
	InsertGuiPair(unknown_error,			"Ocurrio un error desconocido");
	InsertGuiPair(cant_find_help,			"No se encuentran los ficheros de ayuda");
	InsertGuiPair(cant_open_project_page,   "No se puede abrir la pagina dle proyecto");	

}




